local addonName, L = ...

GlootOptions = {}
GlootOptions.panel = CreateFrame("Frame", L["GrindLootOptions"], UIParent, "InsetFrameTemplate")
GlootOptions.events = {}

-- *****************************************************************************************************************

OriginalLoot = LootSlot;
function AutoConfirmBoP_LootSlot(slot)
    OriginalLoot(slot)
    ConfirmLootSlot(slot)
end

function GlootOptions.getBagItems()
    local isinbag = {};

    for bagID = 0, 4 do
        --local bagCount = C_Container.GetContainerNumSlots(bagID)
		local bagCount = GetContainerNumSlots(bagID)
        if bagCount > 0 then
            for slotID = 1, bagCount do
                local bagItemID = GetContainerItemID(bagID, slotID)
                --local info = C_Container.GetContainerItemInfo(bagID, slotID)
                --local bagItemID = info and info.itemID;

                if bagItemID ~= nil then
                    isinbag[bagItemID] = true
                end
            end
        end
    end

    return isinbag
end

function GlootOptions.events:LOOT_OPENED(autoLoot)
    if (GrindLootOptions["Enabled"] ~= true) then
        return
    end

    if (autoLoot == 1) then
        return
    end

    if (GetOptOutOfLoot() == 1) then
        return
    end

    LootIt()
end

function LootIt()
    local isinbag = GlootOptions.getBagItems()

    local cnt = GetNumLootItems()
    local isFishingLoot = IsFishingLoot()
    local _, skillLineProf1, skillLineProf2

    local prof1, prof2, archaeology, fishing, cooking, firstAid = GetProfessions()
    if (prof1) then
        _, _, _, _, _, _, skillLineProf1 = GetProfessionInfo(prof1)
    end
    if (prof2) then
        _, _, _, _, _, _, skillLineProf2 = GetProfessionInfo(prof2)
    end

    local minimumValue = (GrindLootOptions["MinimumValue"] * 10000);

    for lootSlot = 1, cnt do
        local canloot = false

        local lootType = GetLootSlotType(lootSlot)
        if ((lootType == LOOT_SLOT_MONEY) or (lootType == LOOT_SLOT_CURRENCY)) then
            --money or other currency item
            canloot = true;
            if (GrindLootDebug) then
                print("currency")
            end
        end

        if ((canloot == false) and (LootSlotHasItem(lootSlot))) then
            local itemID, itemLink2, itemName, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, armorSlot, textureID, vendorPrice, itemCategory, categoryVersion
            --local itemID, itemType, itemSubType, itemEquipLoc, iconFileDataID, itemClassID, itemSubClassID
            local itemEquipLoc, iconFileDataID, itemClassID, itemSubClassID
            local itemLink = GetLootSlotLink(lootSlot)

            if (itemLink) then
                if (GrindLootDebug) then
                    print(itemLink)
                end

                itemName, itemLink2, itemRarity, itemLevel, itemMinLevel, itemType, itemSubType, itemStackCount, armorSlot, textureID, vendorPrice, itemCategory, categoryVersion = GetItemInfo(itemLink)
                itemID, itemType, itemSubType, itemEquipLoc, iconFileDataID, itemClassID, itemSubClassID = GetItemInfoInstant(itemLink)

                if (itemID ~= nil) then
                    itemID = tonumber(itemID)
                else
                    itemID = 0
                end

                if (GrindLootDebug) then
                    print("itemClassID=", itemClassID)
                    print("itemSubClassID=", itemSubClassID)
                    print("ALL=", itemID, itemType, itemSubType, itemEquipLoc, iconFileDataID, itemClassID, itemSubClassID)
                end

                local sellToVendorPrice = vendorPrice;
                if sellToVendorPrice == nil then
                    sellToVendorPrice = 0;
                else
                    sellToVendorPrice = sellToVendorPrice * itemStackCount;
                end

                if (canloot == false) and (isFishingLoot) then
                    if (itemClassID == GL_LE_ITEM_CLASS_TRADEGOODS) and (itemSubClassID == GLOOT_MEAT_COOKING) then
                        canloot = true
                        if (GrindLootDebug) then
                            print("fishing loot and is a fish or other cooking")
                        end
                    end
                    if (itemClassID == GL_LE_ITEM_CLASS_CONSUMABLE) and (itemSubClassID == GLOOT_CONSUMABLE_OTHER) then
                        canloot = true
                        if (GrindLootDebug) then
                            print("fishing loot and is a bait or other consumable")
                        end
                    end
                end

                if (canloot == false) and (itemClassID == GL_LE_ITEM_CLASS_QUESTITEM) then
                    canloot = true; --quest item
                    if (GrindLootDebug) then
                        print("quest item")
                    end
                end

                if (canloot == false) and (itemClassID == GL_LE_ITEM_CLASS_BATTLEPET) then
                    canloot = true; --pet
                    if (GrindLootDebug) then
                        print("pet")
                    end
                end

                if (canloot == false) and (itemClassID == GL_LE_ITEM_CLASS_GLYPH) then
                    canloot = true; --glyph
                    if (GrindLootDebug) then
                        print("glyph")
                    end
                end

                if (canloot == false) and (itemClassID == GL_LE_ITEM_CLASS_ITEM_ENHANCEMENT) then
                    canloot = true; --enchanted scroll
                    if (GrindLootDebug) then
                        print("enchanted scroll")
                    end
                end

                if (canloot == false) and (itemClassID == GL_LE_ITEM_CLASS_KEY) then
                    canloot = true; --it's a key
                    if (GrindLootDebug) then
                        print("key")
                    end
                end

                if (canloot == false) and (itemClassID == GL_LE_ITEM_QUALITY_ARTIFACT) then
                    canloot = true; --artifacts
                    if (GrindLootDebug) then
                        print("artifact")
                    end
                end

                if (canloot == false) and (itemClassID == GL_LE_ITEM_CLASS_TRADEGOODS) and (GrindLootOptions["GatheringMode"] == true) then
                    canloot = true; --reagent
                    if (GrindLootDebug) then
                        print("reagent and gathering mode")
                    end
                end

                if (canloot == false) and (itemClassID == GL_LE_ITEM_CLASS_TRADEGOODS) then
                    if (skillLineProf1) and (GLOOT_TRADE_SKILL_LOOT[skillLineProf1]) and (GLOOT_TRADE_SKILL_LOOT[skillLineProf1][itemSubClassID]) then
                        canloot = true; --reagent for a skill we have
                        if (GrindLootDebug) then
                            print("reagent for 1st primatry profession")
                        end
                    end
                    if (skillLineProf2) and (GLOOT_TRADE_SKILL_LOOT[skillLineProf2]) and (GLOOT_TRADE_SKILL_LOOT[skillLineProf2][itemSubClassID]) then

                        canloot = true; --reagent for a skill we have
                        if (GrindLootDebug) then
                            print("reagent for 2nd primary profession")
                        end
                    end
                    if (cooking) and (GLOOT_TRADE_SKILL_LOOT[GLOOT_COOKING]) and (GLOOT_TRADE_SKILL_LOOT[GLOOT_COOKING][itemSubClassID]) then
                        canloot = true; --reagent for cooking - and we know cooking - we like food
                        if (GrindLootDebug) then
                            print("reagent for cooking")
                        end
                    end
                end

                if (canloot == false) and (itemClassID == GL_LE_ITEM_CLASS_GEM) then
                    canloot = true; --gem, artifact relic
                    if (GrindLootDebug) then
                        print("gem stuff")
                    end
                end

                if (canloot == false) and ((itemClassID == GL_LE_ITEM_CLASS_MISCELLANEOUS) and (itemSubClassID >= GL_LE_ITEM_QUALITY_EPIC)) then
                    canloot = true; --archeology gorian artifact - other epic_level stuff
                    if (GrindLootDebug) then
                        print("epic junk")
                    end
                end

                if (canloot == false) and (itemClassID == GL_LE_ITEM_CLASS_RECIPE) then
                    canloot = true; --recipe
                    if (GrindLootDebug) then
                        print("recipe")
                    end
                end

                if (canloot == false) then
                    if (GemChip[itemID]) then
                        canloot = true; --gem chip
                        print(GemChip[itemID] .. " " .. L["Gem Chip"] .. " |T" .. textureID .. ":24|t") --special display of color for gem chips
                    end
                end

                if (canloot == false) then
                    --local _, _, _, _, itemRarity = GetLootSlotInfo(lootSlot)
                    --print("itemRarity =",itemRarity);

                    if (itemRarity) and (GrindLootOptions["MinimumRarity"] > 0) and (itemRarity >= GrindLootOptions["MinimumRarity"]) then
                        canloot = true; --rarity based loot
                        if (GrindLootDebug) then
                            print("rarity level")
                        end
                    end

                    if (GrindLootOptions["LootValuable"]) and (GrindLootOptions["MinimumValue"] > 0) and (sellToVendorPrice >= minimumValue) then
                        canloot = true; --vendor value based loot
                        if (GrindLootDebug) then
                            print('valuable')
                            print(itemLink2, ' ', minimumValue, '<', sellToVendorPrice)
                        end
                    end
                end

                if (canloot == false) then
                    if (isinbag[itemID]) then
                        canloot = true; --loot if it is already in bags
                        if (GrindLootDebug) then
                            print("have in bag already")
                        end
                    end
                end

                if (canloot == false) then
                    --badly marked quest items
                    local tip = myTooltipFromTemplate or CreateFrame("GAMETOOLTIP", "myTooltipFromTemplate", nil, "GameTooltipTemplate")
                    tip:UnregisterAllEvents()
                    tip:SetOwner(WorldFrame, "ANCHOR_NONE")
                    tip:SetHyperlink(itemLink)
                    local lines = tip:NumLines()
                    for i = 1, lines do
                        local tooltipline = _G["myTooltipFromTemplateTextLeft" .. i]:GetText();
                        if ((tooltipline == L["This Item Begins a Quest"]) or (tooltipline == L["Quest Item"]) or (tooltipline == L["Binds when picked up"])) then
                            canloot = true;
                            if (GrindLootDebug) then
                                print("badly tagged quest item")
                            end
                            break ;
                        end
                    end
                end

                if (canloot == false) then
                    if (GrindLootOptions["GatheringModeNoJunk"] == true) then
                        throwAway[itemID] = itemID
                        canloot = true
                        if (GrindLootDebug) then
                            print(itemLink2, " is junk but throw away activated")
                        end
                    end

                    if (canloot == false) then
                        if (GrindLootDebug) then
                            --this is either junk or a missed item that should be auto-looted?
                            print("itemRarity=", itemRarity)
                            print("min=", GrindLootOptions["MinimumRarity"])
                            print("LootValuable=", GrindLootOptions["LootValuable"])
                            print("sellToVendorPrice=", sellToVendorPrice)
                            print("min=", minimumValue)
                            print("vendorPrice=", vendorPrice)
                            print("itemStackCount=", itemStackCount)

                            if (itemRarity > 0) then
                                print("itemID=", itemID)
                                print("itemClassID=", itemClassID)
                                print("itemSubClassID=", itemSubClassID)
                                print("ALL=", itemID, itemType, itemSubType, itemEquipLoc, iconFileDataID, itemClassID, itemSubClassID, vendorPrice, itemStackCount)
                            end
                        end
                    end
                end
            else
                print(L["No link for lootType "], lootType);
            end
        end

        if (canloot == true) then
            LootSlot(lootSlot);
        end
    end

    if ((GrindLootOptions["CloseLootWindow"]) or isFishingLoot) then
        CloseLoot()
        if (GrindLootDebug) then
            print("autoclose loot window")
        end
    end
end


-- *****************************************************************************************************************

function GlootOptions.events:ADDON_LOADED(loadedAddonName)
    if (loadedAddonName == "GrindLoot") then
        GlootOptions.panel:UnregisterEvent("ADDON_LOADED");

        GMessage("v" .. DefaultGrindLootOptions["Version"] .. " " .. L["Loaded"])

        if (GrindLootOptions == nil) then
            GrindLootOptions = {}
            for k, v in pairs(DefaultGrindLootOptions) do
                GrindLootOptions[k] = v;
            end
            LootSlot = OriginalLoot;
            return ;
        end

        if ((GrindLootOptions["Version"] == nil) or (GrindLootOptions["Version"] < DefaultGrindLootOptions["Version"])) then
            for k, v in pairs(DefaultGrindLootOptions) do
                GrindLootOptions[k] = v;
            end
            GMessage(L["Old options detected - they have been set to default values"]);
            LootSlot = OriginalLoot;
        end

        if (GrindLootOptions["AutoConfirm"] == true) then
            LootSlot = AutoConfirmBoP_LootSlot;
        else
            GrindLootOptions["AutoConfirm"] = false
        end

        SLASH_GLOOT1 = "/gloot"
        SLASH_GLOOT2 = "/grindloot"
        SLASH_GLOOT3 = "/gl"
        SlashCmdList["GLOOT"] = function(msg)
            GlootCommandHandler(msg)
        end

        GlootOptions.config = GlootOptions:createGlootOptionsPanel()
    end
end

function GlootCommandHandler(msg)
    if (not msg or msg == "") then
        msg = "config"
    end
    local cmd, params = strsplit(" ", msg:lower())
    if (cmd == "config") then
        if (InCombatLockdown()) then
            return
        else
            InterfaceOptionsFrame_OpenToCategory("GrindLoot v" .. DefaultGrindLootOptions["Version"])
        end
    end
    if (cmd == "debug") then
        GrindLootDebug = GrindLootDebug == false
        if (GrindLootDebug) then
            GMessage("Debug mode is ON")
        else
            GMessage("Debug mode is OFF")
        end
    end
end

function GlootOptions:createGlootOptionsPanel()
    local category, layout = Settings.RegisterVerticalLayoutCategory("GrindLoot v" .. DefaultGrindLootOptions["Version"]);
    Settings.RegisterAddOnCategory(category);

    SetupGrindLootCheckBox(category, "Enabled", L["Enabled"], L["Enable or disable GrindLoot"]);

    SetupGrindLootCheckBox(category, "GatheringMode", L["GatheringMode"], L["Enable or disable Gathering mode"]);

    do
        local function GetOptions()
            local container = Settings.CreateControlTextContainer();
            for idx = 0, 4, 1 do
                container:Add(idx, RarityNames[idx], RarityNames[idx])
            end
            return container:GetData();
        end
        SetupGrindLootDropDown(category, "MinimumRarity", Settings.VarType.Number, GetOptions, L["Rarity Looting"], L["Rarity Looting"]);
    end

    do
        local function GetMinGoldValueFormatter()
            return function(value)
                return value .. ' gold'
            end
        end

        local cbSetting = RegisterGrindLootSetting(L["LootValuable"], "LootValuable", "boolean");
        local sliderSetting = RegisterGrindLootSetting(L["MinimumValue"], "MinimumValue", "number");

        local minValue, maxValue, step = 1, 100, 1;
        local options = Settings.CreateSliderOptions(minValue, maxValue, step);
        options:SetLabelFormatter(MinimalSliderWithSteppersMixin.Label.Right, GetMinGoldValueFormatter());

        local initializer = CreateSettingsCheckBoxSliderInitializer(
                cbSetting, L["LootValuable"], L["Enable or disable looting valuable trash items"],
                sliderSetting, options, L["MinimumValue"], L["MinimumValueTooltip"]);
        layout:AddInitializer(initializer);
    end

    SetupGrindLootCheckBox(category, "AutoConfirm", L["AutoConfirm"], L["Automatically confirm BoP items"]);

    SetupGrindLootCheckBox(category, "CloseLootWindow", L["CloseLootWindow"], L["Automatically close the loot window after autolooting even if it is not empty"]);
end

GlootOptions.panel:SetScript("OnEvent", function(self, event, ...)
    if (GlootOptions.events[event]) then
        GlootOptions.events[event](self, ...)
    end
end);
for k, v in pairs(GlootOptions.events) do
    GlootOptions.panel:RegisterEvent(k);
end
